@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2004-2020, University of Amsterdam
+    Copyright (c)  2004-2024, University of Amsterdam
                               VU University Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -78,7 +79,7 @@ free_nb_linkval_name(atom_t name)
 static void
 free_nb_linkval_value(word value)
 { if ( isAtom(value) )
-    PL_unregister_atom(value);
+    PL_unregister_atom(word2atom(value));
   else if ( storage(value) == STG_GLOBAL )
   { GET_LD
     LD->gvar.grefs--;
@@ -87,7 +88,7 @@ free_nb_linkval_value(word value)
 
 
 static void
-free_nb_linkval_symbol(void *name, void* value)
+free_nb_linkval_symbol(table_key_t name, table_value_t value)
 { free_nb_linkval_value((word)value);
   free_nb_linkval_name((atom_t)name);
 }
@@ -103,7 +104,7 @@ new_gvar(DECL_LD atom_t name, atom_t value)
     LD->gvar.nb_vars->free_symbol = free_nb_linkval_symbol;
   }
 
-  addNewHTable(LD->gvar.nb_vars, (void*)name, (void*)old);
+  addNewHTable(LD->gvar.nb_vars, (table_key_t)name, (table_value_t)old);
   PL_register_atom(name);
 
   return old;
@@ -114,11 +115,27 @@ new_gvar(DECL_LD atom_t name, atom_t value)
 static word
 lookup_gvar(DECL_LD atom_t name)
 { if ( LD->gvar.nb_vars )
-    return (word)lookupHTable(LD->gvar.nb_vars, (void*)name);
+    return (word)lookupHTable(LD->gvar.nb_vars, (table_key_t)name);
 
   return 0;
 }
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Get a global variable value.   Flags is reserved to do/do not allow for
+lazy definition of the variable.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+int
+PL_getval(atom_t name, term_t val, unsigned int flags)
+{ GET_LD
+  word w = lookup_gvar(name);
+  if ( w )
+  { *valTermRef(val) = w;
+    return true;
+  }
+
+  return false;
+}
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -144,7 +161,7 @@ setval(DECL_LD term_t var, term_t value, int backtrackable)
   if ( !hasGlobalSpace(3) )		/* also ensures trail for */
   { int rc;				/* TrailAssignment() */
 
-    if ( (rc=ensureGlobalSpace(3, ALLOW_GC)) != TRUE )
+    if ( (rc=ensureGlobalSpace(3, ALLOW_GC)) != true )
       return raiseStackOverflow(rc);
   }
 
@@ -170,7 +187,7 @@ setval(DECL_LD term_t var, term_t value, int backtrackable)
   if ( w == old )
     succeed;
   if ( isAtom(old) )
-    PL_unregister_atom(old);
+    PL_unregister_atom(word2atom(old));
 
   if ( backtrackable )
   { Word p;
@@ -183,7 +200,8 @@ setval(DECL_LD term_t var, term_t value, int backtrackable)
       freezeGlobal();		/* The value location must be */
       if ( storage(old) != STG_GLOBAL )	/* preserved */
 	LD->gvar.grefs++;
-      updateHTable(LD->gvar.nb_vars, (void*)name, (void*)makeRefG(p));
+      updateHTable(LD->gvar.nb_vars,
+		   (table_key_t)name, (table_value_t)makeRefG(p));
     }
 
     TrailAssignment(p);
@@ -192,13 +210,13 @@ setval(DECL_LD term_t var, term_t value, int backtrackable)
   { if ( storage(old) == STG_GLOBAL )
       LD->gvar.grefs--;
 
-    updateHTable(LD->gvar.nb_vars, (void*)name, (void*)w);
+    updateHTable(LD->gvar.nb_vars, (table_key_t)name, (table_value_t)w);
 
     if ( storage(w) == STG_GLOBAL )
     { freezeGlobal();
       LD->gvar.grefs++;
     } else if ( isAtom(w) )
-      PL_register_atom(w);
+      PL_register_atom(word2atom(w));
   }
 
   succeed;
@@ -222,6 +240,10 @@ auto_define_gvar(atom_t name)
   if ( !GD->procedures.exception3 )
     GD->procedures.exception3 = PL_predicate("exception", 3, "user");
 
+  DEBUG(MSG_GVAR_LAZY, Sdprintf("[%d]: auto_define_gvar(%s)\n",
+				PL_thread_self(), PL_atom_chars(name)));
+
+
   if ( !(fid = PL_open_foreign_frame()) )
     return gvar_error;
   av = PL_new_term_refs(3);
@@ -243,6 +265,10 @@ auto_define_gvar(atom_t name)
 
   PL_close_foreign_frame(fid);
 
+  DEBUG(MSG_GVAR_LAZY,
+	Sdprintf("  %s --> %d\n", PL_atom_chars(name), rc));
+
+
   return rc;
 }
 
@@ -260,13 +286,13 @@ int
 gvar_value(DECL_LD atom_t name, Word p)
 { if ( LD->gvar.nb_vars )
   { word w;
-    if ( (w = (word)lookupHTable(LD->gvar.nb_vars, (void*)name)) )
+    if ( (w = (word)lookupHTable(LD->gvar.nb_vars, (table_key_t)name)) )
     { *p = w;
-      return TRUE;
+      return true;
     }
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -305,12 +331,12 @@ getval(DECL_LD term_t var, term_t value, int raise_error)
     switch(auto_define_gvar(name))
     { case gvar_fail:
 	new_gvar(name, ATOM_no_value);
-	return FALSE;
+	return false;
       case gvar_retry:
 	continue;
       case gvar_error:
 	if ( exception_term )
-	  return FALSE;				/* error from handler */
+	  return false;				/* error from handler */
         new_gvar(name, ATOM_no_value);
         goto error;
     }
@@ -321,7 +347,7 @@ error:
     return PL_error(NULL, 0, NULL, ERR_EXISTENCE,
 		    ATOM_variable, var);
   else
-    return FALSE;
+    return false;
 }
 
 
@@ -329,7 +355,7 @@ static
 PRED_IMPL("nb_linkval", 2, nb_linkval, 0)
 { PRED_LD
 
-  return setval(A1, A2, FALSE);
+  return setval(A1, A2, false);
 }
 
 
@@ -337,7 +363,7 @@ static
 PRED_IMPL("nb_getval", 2, nb_getval, 0)
 { PRED_LD
 
-  return getval(A1, A2, TRUE);
+  return getval(A1, A2, true);
 }
 
 
@@ -345,14 +371,14 @@ static
 PRED_IMPL("b_setval", 2, b_setval, 0)
 { PRED_LD
 
-  return setval(A1, A2, TRUE);
+  return setval(A1, A2, true);
 }
 
 static
 PRED_IMPL("b_getval", 2, b_getval, 0)
 { PRED_LD
 
-  return getval(A1, A2, TRUE);
+  return getval(A1, A2, true);
 }
 
 
@@ -362,7 +388,7 @@ PRED_IMPL("nb_delete", 1, nb_delete, 0)
   atom_t name;
 
   if ( !PL_get_atom_ex(A1, &name) )
-    return FALSE;
+    return false;
 
   if ( LD->gvar.nb_vars )
   { word w;
@@ -370,7 +396,8 @@ PRED_IMPL("nb_delete", 1, nb_delete, 0)
     if ( (w = lookup_gvar(name)) )
     { if ( w != ATOM_no_value )
       { free_nb_linkval_value(w);
-	updateHTable(LD->gvar.nb_vars, (void*)name, (void*)ATOM_no_value);
+	updateHTable(LD->gvar.nb_vars,
+		     (table_key_t)name, (table_value_t)ATOM_no_value);
       }
     } else
     { new_gvar(name, ATOM_no_value);
@@ -385,14 +412,12 @@ static
 PRED_IMPL("nb_current", 2, nb_current, PL_FA_NONDETERMINISTIC)
 { PRED_LD
   TableEnum e;
-  atom_t name;
-  word val;
   fid_t fid;
 
   switch( CTX_CNTRL )
   { case FRG_FIRST_CALL:
       if ( PL_is_atom(A1) )
-	return getval(A1, A2, FALSE);
+	return getval(A1, A2, false);
       if ( !PL_is_variable(A1) )
 	return PL_type_error("atom", A1);
       if ( LD->gvar.nb_vars )
@@ -415,10 +440,16 @@ PRED_IMPL("nb_current", 2, nb_current, PL_FA_NONDETERMINISTIC)
 
   if ( !(fid = PL_open_foreign_frame()) )
   { freeTableEnum(e);
-    return FALSE;
+    return false;
   }
-  while( advanceTableEnum(e, (void**)&name, (void**)&val) )
-  { if ( !is_gval(val) )
+
+  table_key_t tk;
+  table_value_t tv;
+  while( advanceTableEnum(e, &tk, &tv) )
+  { atom_t name = (atom_t)tk;
+    word val = (word)tv;
+
+    if ( !is_gval(val) )
       continue;
     if ( PL_unify_atom(A1, name) &&
 	 unify_ptrs(valTermRef(A2), &val, 0) )
